# # collect interval
# interval = 15

# # whether collect protocol stats on Linux
# collect_protocol_stats = false

# # setting interfaces will tell categraf to gather these explicit interfaces
# interfaces = ["eth0"]
 
# enable_loopback_stats=true
# enable_link_down_stats=true
